@@ -1,9 +1,14 @@
 from __future__ import annotations
 
+from time import time
+
+import prometheus_client
 import toolz
+from prometheus_client.core import CounterMetricFamily, GaugeMetricFamily
 
 from distributed.http.prometheus import PrometheusCollector
 from distributed.http.scheduler.prometheus.semaphore import SemaphoreMetricCollector
+from distributed.http.scheduler.prometheus.stealing import WorkStealingMetricCollector
 from distributed.http.utils import RequestHandler
 from distributed.scheduler import ALL_TASK_STATES, Scheduler
 
@@ -14,23 +19,22 @@ class SchedulerMetricCollector(PrometheusCollector):
         self.subsystem = "scheduler"
 
     def collect(self):
-        from prometheus_client.core import CounterMetricFamily, GaugeMetricFamily
 
         yield GaugeMetricFamily(
             self.build_name("clients"),
-            "Number of clients connected.",
+            "Number of clients connected",
             value=len([k for k in self.server.clients if k != "fire-and-forget"]),
         )
 
         yield GaugeMetricFamily(
             self.build_name("desired_workers"),
-            "Number of workers scheduler needs for task graph.",
+            "Number of workers scheduler needs for task graph",
             value=self.server.adaptive_target(),
         )
 
         worker_states = GaugeMetricFamily(
             self.build_name("workers"),
-            "Number of workers known by scheduler.",
+            "Number of workers known by scheduler",
             labels=["state"],
         )
         worker_states.add_metric(["connected"], len(self.server.workers))
@@ -40,7 +44,7 @@ class SchedulerMetricCollector(PrometheusCollector):
 
         tasks = GaugeMetricFamily(
             self.build_name("tasks"),
-            "Number of tasks known by scheduler.",
+            "Number of tasks known by scheduler",
             labels=["state"],
         )
 
@@ -62,8 +66,9 @@ class SchedulerMetricCollector(PrometheusCollector):
             self.build_name("tasks_forgotten"),
             (
                 "Total number of processed tasks no longer in memory and already "
-                "removed from the scheduler job queue. Note task groups on the "
-                "scheduler which have all tasks in the forgotten state are not included."
+                "removed from the scheduler job queue\n"
+                "Note: Task groups on the scheduler which have all tasks "
+                "in the forgotten state are not included."
             ),
             value=task_counter.get("forgotten", 0.0),
         )
@@ -84,16 +89,35 @@ class SchedulerMetricCollector(PrometheusCollector):
                 prefix_state_counts.add_metric([tp.name, state], count)
         yield prefix_state_counts
 
+        now = time()
+        max_tick_duration = max(
+            self.server._max_tick_duration, now - self.server._last_tick
+        )
+        self.server._max_tick_duration = 0
+        yield GaugeMetricFamily(
+            self.build_name("tick_duration_maximum_seconds"),
+            "Maximum tick duration observed since Prometheus last scraped metrics",
+            value=max_tick_duration,
+        )
 
-COLLECTORS = [SchedulerMetricCollector, SemaphoreMetricCollector]
+        yield CounterMetricFamily(
+            self.build_name("tick_count_total"),
+            "Total number of ticks observed since the server started",
+            value=self.server._tick_counter,
+        )
+
+
+COLLECTORS = [
+    SchedulerMetricCollector,
+    SemaphoreMetricCollector,
+    WorkStealingMetricCollector,
+]
 
 
 class PrometheusHandler(RequestHandler):
     _collectors = None
 
     def __init__(self, *args, dask_server=None, **kwargs):
-        import prometheus_client
-
         super().__init__(*args, dask_server=dask_server, **kwargs)
 
         if PrometheusHandler._collectors:
@@ -111,7 +135,5 @@ class PrometheusHandler(RequestHandler):
             prometheus_client.REGISTRY.register(instantiated_collector)
 
     def get(self):
-        import prometheus_client
-
         self.write(prometheus_client.generate_latest())
         self.set_header("Content-Type", "text/plain; version=0.0.4")
